%% SCRIPT FOR DATASET CREATION FOR RAMAN MAP DATA
% Julia Gala de Pablo
% PhD Student
% University of Leeds

% WDF FILE IMPORTING:
% Cosmic rays should be corrected from the wdf files before using this
% script.

clear all
close all
clc

% Add one dirPath per dataset:
dirPath={...   
    'C:\DirPathExample';...
    };

% Saving Path is common for all datasets:
savingPath='C:\SavingPathExample';

% For baseline subtraction, add the path where the function
% f_baseline_corr is (Koch et al, 2017) and the WiRe import functions are:      
addpath('C:\MatlabFunctionsPath')

CalibrationPeakB=520.5; % Silicon reference ideal peak position

% Map 1
Name{1}='Map1';
CalibrationPeak{1}=520.5;
NameSaving{1}='Map1_';

% Datasets to run:
selectedDataset=[1];

%% Optimizing the parameters
% This section will give a summary plot of what one of the spectra would 
% look like after the different stages of Dataset Creation, to choose the
% right parameters before running the whole script (that might take a while)
% RUN THIS SECTION BY ITSELF UNTIL HAPPY WITH THE PARAMETERS, then move to
% the next section

datasettesting=1; % Choose which dataset you are testing
cd(dirPath{datasettesting});
indextesting=20; % Choose one spectrum to test on.
% Creating the structure file with the map inside
MapTesting=WdfReader([Name{datasettesting} '.wdf']);
% Obtaining the spectra from the structure file, where each row is a
% spectrum
RamanSpectratesting=MapTesting.GetSpectra(1,MapTesting.Count);
% Changing it to columns because that is how I usually work with them
RamanSpectratesting=RamanSpectratesting';
Wavenumbers=MapTesting.GetXList;
CalibrationPeaktesting=CalibrationPeak{datasettesting};
% Choose the final Xintvalues that you want to use for the spline
% interpolation. For example:
%           Xintvalues=[700:1500]
% Make sure you choose values withing the range you have acquired, or the
% spline fitting will go very weird.
Xintvalues=[725:1800];

RamanSpectraInttesting=spline(Wavenumbers+(520.5-CalibrationPeak{datasettesting}),...
    RamanSpectratesting(:,indextesting),Xintvalues);

% Choose the parameters for the baseline subtraction
smoothwidth=50;
bwidth=400;
iterations=10;
[RamanSpectraBaselinetesting, baseline]=f_baseline_corr(Xintvalues,...
        RamanSpectraInttesting,smoothwidth,bwidth,iterations); 
% Plot the whole thing so you get to change the parameters :)

figure
subplot(3,1,1)
title('Raw data')
hold on
plot(Wavenumbers,RamanSpectratesting(:,indextesting))
xlabel('Wavenumber / cm^{-1}')
ylabel('Raman Intensity / counts')
axis tight

subplot(3,1,2)
title('Interpolated data')
hold on
plot(Xintvalues,RamanSpectraInttesting)
plot(Xintvalues,baseline,'r')
xlabel('Wavenumber / cm^{-1}')
ylabel('Raman Intensity / counts')
legend('Interpolated Raman Spectra','Baseline fitted')
axis tight

subplot(3,1,3)
title('Baseline subtracted data')
hold on
plot(Xintvalues,RamanSpectraBaselinetesting)
xlabel('Wavenumber / cm^{-1}')
ylabel('Raman Intensity / counts')
axis tight

MapTesting.Close
clear MapTesting

%% Loading the data
fprintf('Loading the Data \n')
tic

for j=selectedDataset
    cd(savingPath)
    try load([NameSaving{j} '_datasetraw' '.mat'])
    catch
        warning('Raw dataset could not be found, proceeding to load from wdf file');
    end
    fprintf(['Loading data from dataset' num2str(j) '\n']);
    cd(dirPath{j});
    % Creating the structure file with the map inside
    Map=WdfReader([Name{j} '.wdf']);    
    if exist('RamanSpectra')==0
        % Obtaining the spectra from the structure file, where each row is a
        % spectrum
        RamanSpectra=Map.GetSpectra(1,Map.Count);
        % Changing it to columns because that is how I usually work with them
        RamanSpectra=RamanSpectra';
        Wavenumbers=Map.GetXList;
        CalibrationPeakTemp=CalibrationPeak{j};
        cd(savingPath)
        save([NameSaving{j} '_datasetraw'],...
            'RamanSpectra','Wavenumbers','CalibrationPeakTemp')
    end
    toc

%% X alignment of the spectra respect to the Calibration peak position in Si

% Every day of measuring a calibrating spectra is taken and it is aligned
% as good as it can to the 520.5 cm^-1 peak of Silicon. However, sometimes
% the peak seems to be around 520.3 or 520.8. This peak will be used to
% align the Shift of the spectra and reduce the variability between days.

% As the PCA only considers the Y-values for analysis,
% a spline function should be fit and the Y-values should be guarantee to
% have exactly the same X.
fprintf('Aligning spectra and fitting to the selected X values using a spline \n')
tic
RamanSpectraInt=NaN(length(Xintvalues),Map.Count);

for k=1:Map.Count
   RamanSpectraInt(:,k)=spline(Wavenumbers+(520.5-CalibrationPeak{j}),RamanSpectra(:,k),Xintvalues);
end
toc

clear RamanSpectra Wavenumbers

%% Optional: you can perform EMSC correction here - use EMSC function

%% Subtract the baseline
fprintf('Subtracting the baseline \n')

tic
RamanSpectraBaseline=NaN(size(RamanSpectraInt));
for i=1:size(RamanSpectraInt,2)
    if mod(i,20)==0
        disp(['Baseline correcting spectrum ' num2str(i) ' of ' num2str(Map.Count)])
    end
    % First step to remove the maximum possible of the baseline
    if i==1
        tic
    end
    [RamanSpectraBB, ~]=f_baseline_corr(Xintvalues,...
        RamanSpectraInt(:,i),smoothwidth,bwidth,iterations);%,'plot2D');    
    RamanSpectraBaseline(:,i)=RamanSpectraBB.';
    if i==10
        elapsedTime = toc;
    end
    if mod(i,20)==0
        disp(['Time left ' num2str(round(elapsedTime*(Map.Count-i)/600)) 'min'])
    end
    clear RamanSpectraBB
end
toc
clear RamanSpectraB RamanSpectraInt

%% Data Matrix
fprintf('Creating the Data Matrix \n')
tic
DataMatrix=RamanSpectraBaseline;

clear RamanSpectraBaseline

%% Calculate the Average and STD

Average=mean(DataMatrix,2);
Error=std(DataMatrix,0,2);


%% Obtain the position information from the Filename

WMAP=Map.GetWMapblock;
Location=WMAP.Location;
StepSize=WMAP.StepSize;
MapDim=WMAP.numPoints;
Xposition=(0:(MapDim(1)-1)).*StepSize(1);
Yposition=(0:(MapDim(2)-1)).*StepSize(2);
TimeValue=Map.GetOriginTimes(1,Map.Count);
cd(dirPath{j})

%% Saving Data
    cd(savingPath)
    save([NameSaving{j} '_datasetanalysed'],...
        'DataMatrix','Average','Error','Xposition','Yposition','TimeValue',...
        'Xintvalues','StepSize','Location','MapDim')
    
%% Clear data for next dataset
    clear RamanSpectra* CalibrationPeakTemp...
        DataMatrix RamanSpectraZap
    Map.Close
    clear Map
    status = fclose('all');
        
end

disp('Done! :)')