function [s,background,c_r,c_b,B_N] = EMSC(S,r,b,N)
% EMSC correction for multiple backgrounds (in this case taylored for wax
% and glass, and multiple observations of each of them)
% Originally built based on the initial code supplied by Prof. Hugh Byrne
% Outputs the corrected signal as well as the total background and other
% info. Assumes S,r,and g are all 1D vectors (just spectral irradiance
% values)
%
% [s,background,c_r,c_b,B_N] = EMSC(S,r,b,N)
%
%% INPUTS
%   S is the spectrum to be corrected (no need fo smoothing in advance).
%   Has to be a 1D row spectrum.
%   r is a reference cell spectrum that is free of the glass signal. Can be
%   a matrix, but spectra should be in rows.
%   b is the background spectrum. Can be a matrix, but spectra should be in
%   rows.
%   N is the order to the background polymonial to be included (%Even N = 1,
%   i.e.  straight line often works well). Should be an integrer.
%
%% OUTPUTS
%   s is the returned signal with some weight of glass removed and an N order
%   polymonial removed
%   background is the total backgorund signal (glass + poly)
%   c_r and c_g are the weights of r and g 
%   B_N is the isolated polynomial signal in the background.

% Checking the INPUT
S_size=size(S);
r_size=size(r);
b_size=size(b);
N_size=size(N);
if S_size(1)~=1
    error('ERROR: S is not a row vector')
elseif r_size(2)~=S_size(2)
    error('ERROR: r and S don not have the same number of columns')
elseif b_size(2)~=S_size(2)
    error('ERROR: b and S don not have the same number of columns')
elseif mod(N,1)~=0
    error('ERROR: N needs to be an integrer')
end  

% Polynomial reference spectrum
x = 1:S_size(2); % row vector
mu = [mean(x); std(x)];
x = (x - mu(1))/mu(2); % normalised

% Construct Vandermonde matrix for polynomial subtraction
% In linear algebra, a Vandermonde matrix, is a matrix with the terms of a 
% geometric progression in each row, i.e., an m  n matrix
%
% [ 1 a a^2 ... a^n]
% [ 1 b b^2 ... b^n]
% [ ...            ]
% [ 1 m m^2 ... m^n]
% 
Vtemp = vander(x);
V=Vtemp(:,end-N:end);
% Add to the matrix V the reference and background coefficients
% This is independent on how many references and backgrounds you want to
% add
try
    V = [V , r.' , b.'];
catch
    error('Dimensions of arrays being concatenated are not consistent.')
end
% Solve least squares problem using the function qr from matlab [Q,R] = qr(V)
% where A is m-by-n, produces an m-by-n upper triangular matrix R and an 
% m-by-m unitary matrix Q so that V = Q*R.
% More info in: https://en.wikipedia.org/wiki/QR_decomposition
try [Q,R] = qr(V,0); % QR decomposition of V
catch
    error('Error in the QR decomposition')
end
p = R\(Q'*S'); % Calculate the weights

% OUTPUTS
c_p = p(1:N+1); % weights
B_N=polyval(c_p,x); % Total polynomial contribution
c_r=p(N+2:N+2+r_size(1)-1);
c_b=p(N+2+r_size(1):N+2+r_size(1)+b_size(1)-1); % Weight/s of background
B_b=c_b.'*b; % Calculate the background from the background spectra
background=B_N+B_b; % Calculate the total background
s=(S-background);





